# Resource object code (Python 3)
# Created by: object code
# Created by: The Resource Compiler for Qt version 6.7.2
# WARNING! All changes made in this file will be lost!

from PySide6 import QtCore

qt_resource_data = b"\
\x00\x00#\xcf\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00@\x00\x00\x00@\x08\x06\x00\x00\x00\xaaiq\xde\
\x00\x00\x00\xc5zTXtRaw prof\
ile type exif\x00\x00x\
\xdamP\xdb\x0d\xc3 \x0c\xfc\xf7\x14\x1d\x01?\x02\xf6\
8\xa4I\xa5n\xd0\xf1k\xb0\x13%UO\xe2\xfc8\
\xeb\x00\xc3\xfey\xbf\xe01@( K\xd3j\xb5\x16\
\x87\x98\x18uO\xb4\x04\xfad,2yBR\xf2\xfa\
\xd6\x87S o\xb1G\x8eRk\xce\x1f}<\x0d\x22\
t\xcf\x96\x8b\x91>SX\xef\x82\xe5\x0d\xa4?Fy\
\x11\x8f\x17\x91'[\x1aY\x1a1\x85\x80i\xd0\xe3[\
\xa5\x9a\xb6\xeb\x17\xd6\xbd\xdc\xa1q`\x10\xb7\xe9}\x9a\
\xfc\xd6\xd2|{\xdb\xe2M&\xda\x19\xb983k<\
\x80\xc7\x11\xe0\xee\x899#\x9b\x0f\x22\xb7\x99\x8b3\xcd\
Q\x8c\x85\xfc\xdb\xd3\x01\xf8\x02\xe7}Y+'\xc0\x99\
\xfa\x00\x00\x01\x84iCCPICC pro\
file\x00\x00x\x9c}\x91=H\xc3@\x1c\xc5\
_S\xa5*-\x0a\xed \x22\x92\xa1:\xd9EE\x1c\
K\x15\x8b`\xa1\xb4\x15Zu0\xb9\xf4\x0b\x9a4$\
).\x8e\x82k\xc1\xc1\x8f\xc5\xaa\x83\x8b\xb3\xae\x0e\xae\
\x82 \xf8\x01\xe2\xec\xe0\xa4\xe8\x22%\xfe/)\xb4\x88\
\xf5\xe0\xb8\x1f\xef\xee=\xee\xde\x01B\xa3\xc2T\xb3'\
\x0a\xa8\x9ae\xa4\xe211\x9b[\x15}\xaf\xe8G\x10\
C\x08`Lb\xa6\x9eH/f\xd0u|\xdd\xc3\xc3\
\xd7\xbb\x08\xcf\xea~\xee\xcf\x11P\xf2&\x03<\x22q\
\x94\xe9\x86E\xbcA<\xbbi\xe9\x9c\xf7\x89C\xac$\
)\xc4\xe7\xc4\x93\x06]\x90\xf8\x91\xeb\xb2\xcbo\x9c\x8b\
\x0e\x0b<3ddR\xf3\xc4!b\xb1\xd8\xc1r\x07\
\xb3\x92\xa1\x12\xcf\x10\x87\x15U\xa3|!\xeb\xb2\xc2y\
\x8b\xb3Z\xa9\xb1\xd6=\xf9\x0b\xfdym%\xcdu\x9a\
\xa3\x88c\x09\x09$!BF\x0deT`!B\xab\
F\x8a\x89\x14\xed\xc7\xba\xf8G\x1c\x7f\x92\x5c2\xb9\xca\
`\xe4X@\x15*$\xc7\x0f\xfe\x07\xbf\xbb5\x0b\xd3\
Sn\x92?\x06\xf4\xbe\xd8\xf6\xc78\xe0\xdb\x05\x9au\
\xdb\xfe>\xb6\xed\xe6\x09\xe0}\x06\xae\xb4\xb6\xbf\xda\x00\
\xe6>I\xaf\xb7\xb5\xf0\x110\xb8\x0d\x5c\x5c\xb75y\
\x0f\xb8\xdc\x01\x86\x9ft\xc9\x90\x1c\xc9KS(\x14\x80\
\xf73\xfa\xa6\x1c\x10\xbc\x05\x06\xd6\xdc\xdeZ\xfb8}\
\x002\xd4\xd5\xf2\x0dpp\x08L\x14){\xbd\xcb\xbb\
\xfb:{\xfb\xf7L\xab\xbf\x1f\x97Kr\xb55\xc1\x08\
0\x00\x00\x19kiTXtXML:com\
.adobe.xmp\x00\x00\x00\x00\x00<\
?xpacket begin=\x22\
\xef\xbb\xbf\x22 id=\x22W5M0MpC\
ehiHzreSzNTczkc9\
d\x22?>\x0a<x:xmpmeta \
xmlns:x=\x22adobe:n\
s:meta/\x22 x:xmptk\
=\x22XMP Core 4.4.0\
-Exiv2\x22>\x0a <rdf:R\
DF xmlns:rdf=\x22ht\
tp://www.w3.org/\
1999/02/22-rdf-s\
yntax-ns#\x22>\x0a  <r\
df:Description r\
df:about=\x22\x22\x0a    \
xmlns:xmpMM=\x22htt\
p://ns.adobe.com\
/xap/1.0/mm/\x22\x0a  \
  xmlns:stEvt=\x22h\
ttp://ns.adobe.c\
om/xap/1.0/sType\
/ResourceEvent#\x22\
\x0a    xmlns:stRef\
=\x22http://ns.adob\
e.com/xap/1.0/sT\
ype/ResourceRef#\
\x22\x0a    xmlns:dc=\x22\
http://purl.org/\
dc/elements/1.1/\
\x22\x0a    xmlns:exif\
=\x22http://ns.adob\
e.com/exif/1.0/\x22\
\x0a    xmlns:GIMP=\
\x22http://www.gimp\
.org/xmp/\x22\x0a    x\
mlns:photoshop=\x22\
http://ns.adobe.\
com/photoshop/1.\
0/\x22\x0a    xmlns:ti\
ff=\x22http://ns.ad\
obe.com/tiff/1.0\
/\x22\x0a    xmlns:xmp\
=\x22http://ns.adob\
e.com/xap/1.0/\x22\x0a\
   xmpMM:Documen\
tID=\x22adobe:docid\
:photoshop:89c85\
ad7-1cf9-11ea-99\
59-d7f4e8b866ec\x22\
\x0a   xmpMM:Instan\
ceID=\x22xmp.iid:22\
39ac63-99de-4646\
-a587-eb1d8c25c6\
2b\x22\x0a   xmpMM:Ori\
ginalDocumentID=\
\x22xmp.did:107ce1f\
0-9400-a748-8c5d\
-f3b99246c8a3\x22\x0a \
  dc:format=\x22ima\
ge/png\x22\x0a   exif:\
ApertureValue=\x224\
000000/1000000\x22\x0a\
   exif:Compress\
edBitsPerPixel=\x22\
5/1\x22\x0a   exif:Cus\
tomRendered=\x220\x22\x0a\
   exif:DateTime\
Original=\x222012-0\
3-14T13:05:24\x22\x0a \
  exif:DigitalZo\
omRatio=\x224416/44\
16\x22\x0a   exif:Exif\
Version=\x220221\x22\x0a \
  exif:ExposureB\
iasValue=\x220/3\x22\x0a \
  exif:ExposureM\
ode=\x220\x22\x0a   exif:\
ExposureTime=\x221/\
60\x22\x0a   exif:File\
Source=\x223\x22\x0a   ex\
if:FlashpixVersi\
on=\x220100\x22\x0a   exi\
f:FNumber=\x2240/10\
\x22\x0a   exif:FocalL\
ength=\x2221461/100\
0\x22\x0a   exif:Focal\
PlaneResolutionU\
nit=\x222\x22\x0a   exif:\
FocalPlaneXResol\
ution=\x224416000/2\
92\x22\x0a   exif:Foca\
lPlaneYResolutio\
n=\x223312000/219\x22\x0a\
   exif:MaxApert\
ureValue=\x22128/32\
\x22\x0a   exif:Meteri\
ngMode=\x225\x22\x0a   ex\
if:PixelXDimensi\
on=\x22250\x22\x0a   exif\
:PixelYDimension\
=\x22250\x22\x0a   exif:S\
ceneCaptureType=\
\x220\x22\x0a   exif:Sens\
ingMethod=\x222\x22\x0a  \
 exif:ShutterSpe\
edValue=\x22189/32\x22\
\x0a   exif:WhiteBa\
lance=\x220\x22\x0a   GIM\
P:API=\x222.0\x22\x0a   G\
IMP:Platform=\x22Wi\
ndows\x22\x0a   GIMP:T\
imeStamp=\x22172676\
2470562195\x22\x0a   G\
IMP:Version=\x222.1\
0.38\x22\x0a   photosh\
op:ColorMode=\x223\x22\
\x0a   photoshop:Da\
teCreated=\x222012-\
03-14T13:05:24\x22\x0a\
   photoshop:ICC\
Profile=\x22sRGB IE\
C61966-2.1\x22\x0a   t\
iff:Make=\x22Canon\x22\
\x0a   tiff:Model=\x22\
Canon PowerShot \
G10\x22\x0a   tiff:Ori\
entation=\x221\x22\x0a   \
tiff:ResolutionU\
nit=\x222\x22\x0a   tiff:\
XResolution=\x22720\
000/10000\x22\x0a   ti\
ff:YCbCrPosition\
ing=\x222\x22\x0a   tiff:\
YResolution=\x22720\
000/10000\x22\x0a   xm\
p:CreateDate=\x2220\
12-03-14T13:05:2\
4\x22\x0a   xmp:Creato\
rTool=\x22GIMP 2.10\
\x22\x0a   xmp:Metadat\
aDate=\x222024:09:1\
9T17:14:28+01:00\
\x22\x0a   xmp:ModifyD\
ate=\x222024:09:19T\
17:14:28+01:00\x22>\
\x0a   <xmpMM:Histo\
ry>\x0a    <rdf:Seq\
>\x0a     <rdf:li\x0a \
     stEvt:actio\
n=\x22created\x22\x0a    \
  stEvt:instance\
ID=\x22xmp.iid:107c\
e1f0-9400-a748-8\
c5d-f3b99246c8a3\
\x22\x0a      stEvt:so\
ftwareAgent=\x22Ado\
be Photoshop CC \
2015 (Windows)\x22\x0a\
      stEvt:when\
=\x222012-03-14T13:\
05:24\x22/>\x0a     <r\
df:li\x0a      stEv\
t:action=\x22conver\
ted\x22\x0a      stEvt\
:parameters=\x22fro\
m image/jpeg to \
application/vnd.\
adobe.photoshop\x22\
/>\x0a     <rdf:li\x0a\
      stEvt:acti\
on=\x22saved\x22\x0a     \
 stEvt:changed=\x22\
/\x22\x0a      stEvt:i\
nstanceID=\x22xmp.i\
id:50c42a08-8daa\
-fb4a-af9c-b6348\
bb6a55f\x22\x0a      s\
tEvt:softwareAge\
nt=\x22Adobe Photos\
hop CC 2015 (Win\
dows)\x22\x0a      stE\
vt:when=\x222019-12\
-12T16:07:29Z\x22/>\
\x0a     <rdf:li\x0a  \
    stEvt:action\
=\x22saved\x22\x0a      s\
tEvt:changed=\x22/\x22\
\x0a      stEvt:ins\
tanceID=\x22xmp.iid\
:431c15ae-504c-4\
44a-85f0-08b4ffb\
9fb9b\x22\x0a      stE\
vt:softwareAgent\
=\x22Adobe Photosho\
p CC 2015 (Windo\
ws)\x22\x0a      stEvt\
:when=\x222019-12-1\
2T16:08:06Z\x22/>\x0a \
    <rdf:li\x0a    \
  stEvt:action=\x22\
converted\x22\x0a     \
 stEvt:parameter\
s=\x22from applicat\
ion/vnd.adobe.ph\
otoshop to image\
/png\x22/>\x0a     <rd\
f:li\x0a      stEvt\
:action=\x22derived\
\x22\x0a      stEvt:pa\
rameters=\x22conver\
ted from applica\
tion/vnd.adobe.p\
hotoshop to imag\
e/png\x22/>\x0a     <r\
df:li\x0a      stEv\
t:action=\x22saved\x22\
\x0a      stEvt:cha\
nged=\x22/\x22\x0a      s\
tEvt:instanceID=\
\x22xmp.iid:a1fb072\
1-5ece-3747-b056\
-3fe3f5c5e8c9\x22\x0a \
     stEvt:softw\
areAgent=\x22Adobe \
Photoshop CC 201\
5 (Windows)\x22\x0a   \
   stEvt:when=\x222\
019-12-12T16:08:\
06Z\x22/>\x0a     <rdf\
:li\x0a      stEvt:\
action=\x22saved\x22\x0a \
     stEvt:chang\
ed=\x22/\x22\x0a      stE\
vt:instanceID=\x22x\
mp.iid:75ca8a53-\
f03f-4709-b94d-0\
b607a4ee0ee\x22\x0a   \
   stEvt:softwar\
eAgent=\x22Gimp 2.1\
0 (Windows)\x22\x0a   \
   stEvt:when=\x222\
024-09-19T17:14:\
30\x22/>\x0a    </rdf:\
Seq>\x0a   </xmpMM:\
History>\x0a   <xmp\
MM:DerivedFrom\x0a \
   stRef:documen\
tID=\x22xmp.did:107\
ce1f0-9400-a748-\
8c5d-f3b99246c8a\
3\x22\x0a    stRef:ins\
tanceID=\x22xmp.iid\
:431c15ae-504c-4\
44a-85f0-08b4ffb\
9fb9b\x22\x0a    stRef\
:originalDocumen\
tID=\x22xmp.did:107\
ce1f0-9400-a748-\
8c5d-f3b99246c8a\
3\x22/>\x0a   <exif:Fl\
ash\x0a    exif:Fir\
ed=\x22True\x22\x0a    ex\
if:Function=\x22Fal\
se\x22\x0a    exif:Mod\
e=\x223\x22\x0a    exif:R\
edEyeMode=\x22True\x22\
\x0a    exif:Return\
=\x220\x22/>\x0a   <exif:\
ISOSpeedRatings>\
\x0a    <rdf:Seq>\x0a \
    <rdf:li>200<\
/rdf:li>\x0a    </r\
df:Seq>\x0a   </exi\
f:ISOSpeedRating\
s>\x0a  </rdf:Descr\
iption>\x0a </rdf:R\
DF>\x0a</x:xmpmeta>\
\x0a               \
                \
                \
                \
                \
                \
     \x0a          \
                \
                \
                \
                \
                \
          \x0a     \
                \
                \
                \
                \
                \
               \x0a\
                \
                \
                \
                \
                \
                \
    \x0a           \
                \
                \
                \
                \
                \
         \x0a      \
                \
                \
                \
                \
                \
              \x0a \
                \
                \
                \
                \
                \
                \
   \x0a            \
                \
                \
                \
                \
                \
        \x0a       \
                \
                \
                \
                \
                \
             \x0a  \
                \
                \
                \
                \
                \
                \
  \x0a             \
                \
                \
                \
                \
                \
       \x0a        \
                \
                \
                \
                \
                \
            \x0a   \
                \
                \
                \
                \
                \
                \
 \x0a              \
                \
                \
                \
                \
                \
      \x0a         \
                \
                \
                \
                \
                \
           \x0a    \
                \
                \
                \
                \
                \
                \
\x0a               \
                \
                \
                \
                \
                \
     \x0a          \
                \
                \
                \
                \
                \
          \x0a     \
                \
                \
                \
                \
                \
               \x0a\
                \
                \
                \
                \
                \
                \
    \x0a           \
                \
\x0a<?xpacket end=\x22\
w\x22?>_\x5c,\x10\x00\x00\x00\x06bKGD\
\x00\xff\x00\xff\x00\xff\xa0\xbd\xa7\x93\x00\x00\x00\x09pH\
Ys\x00\x00\x0b\x13\x00\x00\x0b\x13\x01\x00\x9a\x9c\x18\x00\
\x00\x00\x07tIME\x07\xe8\x09\x13\x10\x0e\x1e\x01\xd5\
\x1a\xd2\x00\x00\x07\x84IDATx\xda\xd5\x9bil\
\x9c\xc5\x19\xc7\x7f\xf3\x1e\xbb\x8ew\xbd\xeb\xec\xae\xd9]\
\x13\xaf\x89\xe3\xd8\x0enl\x83\x14\x12'\x86\x90\x0f\xb4\
\xa9T\x95D\xa2\x5c\x91\xa0\xa5\x82R.\x85\xd0\xa2V\
\xa8\xa2\x90\xaa8U\x12!Q@\x08DE?T-\
\x08\x81DC[Q\x08jA\x84\x0f\xa0\x80J9\x1d\
W8U\x12\x0e'&f\xe3\xc4\xc7\xf2\xc1\xb3\xe8e\
\xb3\xc7\xcc\xec\xe5<\xd2Hk\xbf\xef\xcc3\xff9\x9e\
\xfb\x15T\x9f\xda\x80\xd5@\x1f\xd0\x05,\x05\x92@\x18\
X$\xdf\x99\x02\x8e\x01\x87\x81Q\xe0\x03\xe0m\xe0u\
\xe0c\xce0\xb2\x80A`\x18x\x07\x98\x012\x86m\
\x06\xf8/\xb0\x03X\x0b\xd8\x0b\x19x\x04\xd8\x0a\xbc\x07\
\xcc\x95\x01\xbaP\x9b\x03\xde\x07\xb6\x01\xd1\x85\x04|1\
\xf0k`\xbc\x0a\xa0\x0b\xb5\xa3\xc0v\xb9\xe8u#\x07\
\xb8\x018RC\xe0\xb9\xed\x13\xe0\xa7\x80[k\xf0]\
\xc0\xbf\xea\x08<\xb7\xbd\x02\xf4\xd4\x02\xb8\x00\xb6\x00\x13\
\x0b\x08|\xb6}\x01\x5c#\xe7X\x15\xb2\x81]U\x12\
p\x95\x14\x94\xf7\xcb\xeby\x1a\xb5\xb7\xb7GB\xa1\xf0\
\xdd\x8e\xe3\x5c\xa1\x0b\xbe\x01xj\x01\x03\xcfm\xcfx\
l\x0c\x96,Y\x92\x08\x06\x83\xc3\x8e\xe3d\x05\xf5\x83\
\xba\xe0\x9f?\x83\xc0g\xdb?\x92\xc9\xe4\xf2@ \xf0\
\x80\xe38\xc7s\x9e\xbd\xea\xbd\xd3\xa5\x8e\xfd\x9f\x81\xcb\
*p\x85\x8e\x02\xfb\xa4\x85w\x00\xf8\x148!\x9f-\
\x02Z\x80\x0ei1\xae\x91\xea\xd5LP\x89\xafae\
2\x99L>\x8c\x13\x92\xdft)\x81\xb7\xb3\xcc]\xf8\
L\x1e\xb7\x0d\x80_\x03\x83_\xf6yP\x8e\xa1\xc4O\
\x08\x91\x11B\xa8Z\x98]\xa5&\xb1\xa5\x0c\x81w\x10\
\xb8\x1dh\xaa\xc0\xc9i\x92c\x1d,\x0a\xdc\xb2t\xe7\
\xb8\xb9\x18\xd3\xe5\x86\xaa\xee\xa4<5\xa1*h\xa1\x90\
\x1c\xfb\xa4\xc1\x8e\xe7k\xbf*f\xe1\xfd\xdb`\xc0\x03\
\xd2\x09\xaa\xae\xf9\xe98\xebl\xdb\xfe\xa4\x0c\xe0\xd9\xf6\
d!\x1e7\x18\x0c\xf6\xb2\x14*U\xa3\x1bo\xbcI\
\xa4\xda\xda\xd7\xfb\xfd\xfe\x97\x85\x103\x15\xd0\x12\x1f\x16\
rltm\xfb=@\xa0\xda;\xdf\xb9\xacs\xb5e\
Y3\x15T\x93\xd3\xf94\xcd=\x06;_u\xf0}\
+W:M\xc1\xe0\xbeJ[\x8dB\x88\xa1\x5c\x7f~\
\x5c\xf3\xce\xc7\xca\x08\x9a\x84\x0b\x99\xac\xb9tV\xcbY\
wT\x12\xbcm\xdb\xc7\xc2\xe1\xd0\xdd\xf1x<\xec\xe5\
\xb3US\xda\x9b\x0a\xbc\x15\xc0\xdf\xe5\x11< \xaf\xd0\
N\xe0:`\xe5ing\xe7\xf2n\xc7q&+\x01\
<\xab5\x5c\xd7\xfde\xbe\x1dyOc\xb0\x9d\x06\xc0\
\x03\xc0o\x80t\x91q\x8fyUh\xdf\xb7V\xda\xc1\
@`o\xa5\x80{4\xc7G\xb9\xe1\xb5A\x0d\xa3\xe7\
\xff\x06\x06\xce\xa5\x92\xa9\xca\xf8[\xb2\x9d\xceN\xb6\xde\
V.x\xcb\xb2\xf2\xd9\x0as\xc0\x85\xde\x09\xee\xd0\x18\
t\x9bf\xb8\xec\x19M\x8b\xf2o\x00\xa9T\xfb%\x8e\
m~\xf4\x15\x8c\xa4]\xde\x89\xbe\xa3a\xdb\x075\x16\
\xc0\x95am\x9d\xc9\xa7S\xa9s\xce\xf3\xfb\xfd\x07\x0d\
@\xcf\xda\xb6}R\xd1H\xfa\xc0\x1b\xb7W\xd5\xaf\x0f\
\x19\xdc\xfd\xed\x06\xc7\xf6\x94n\x9f\x86\x86\x86\xbd\xd1h\
t\x9d\x10\xe2a\xc5>\xb32G\xc1e\x1a\x8c6\x18\
,@\x8f\x94\xf8Z\x80|>\xf7\x0bE\xe0\xafG#\
\x91o\xf7\xf7\xf7[\x92\xdf\x06\x0d>W\x02\xdc\xab\x11\
\x8a\xf6\x19\xaa\xbeWu\xad\xb4X,\xdawvkk\
_8\x1c\xfaI \x10\xf8\xa3\xcf\xe7\xbek\xdb\xf6\xd7\
\x1a\xc4\xef\xf7\xff'\x12\x89l\xee\xed\xed\xcdM\x96\xf8\
\xe4\x5cU\xf8\xdc\x87\x0cx\xa8\xbc\xfc\xbcq\xaa\xc8\xb2\
\xae5\x10d?\xf7\x8eq\xf5\xd5W\x8aT\xaa-\xb6\
\xb4\xbd}m\x22\x1e\xbfdp\xcd\x9ab\xa1p\xd5\x08\
\xd6\xd3\xc8\xfc\x9b\xca\xcb\xc3\xba\xc0/\xfd\xfe&k\xd5\
\xaa\x0b\xa2\xb1H\xf4Y\x83\x05x\xbb\x0c\xebyX\x91\
\xc7\x9b\x8eLT\xaa\xd0H\xa9\x17\xd6_t\xb1\x95N\
\xa7\x93\x87\x8f\x1c\xba\xf8\xcb/\xd3\x1b^|\xe9\x9f\xab\
\xa6\xa6\xa6\xbagff\xfcB\x88\xd9L&\xa3\x93\xdb\
;W\x86\xc7L\x16bD\xf1\xbd$\x1a\x81\x8f\x82\x11\
\x94\xee\xee\x9e\xa5\x91H\xf4\x91PS\xe8-!\xc4l\
n_\xc7q\xa6\x92\x89\xc4\x0f\x0dN\xc1v\xc3\x13\xb0\
Yq\xfcI\x00U\x95\xb31\x1f\xa7T*\x15s]\
\xb7\xa8\x19\xdd\xd8\xd8\xf8F\xaa\xad\xcd\x95\xbaWg\x01\
F\xa4\x99\xaeK\x1b\x15\xc7?e\x95q\xcf\xe8\xed\xed\
m\x18\x1f\x1f\x7fzzz\xba\xbb\xd8{\x8d\x8d\x8d/\
|<66\x0d\xfcI\x93\xc5\xd2jG\x99,\xe6\x8b\
\x13Th\x91\xf7\x8f\xa1\xb5\xeb\xac\xd1\xd1\xd1\x87''\
'/*\x19\xcc\x0b\x85\xf6\xc8\x9fO\x94\x0aE\xe7\x89\
L_n\x80k\x91\xe2{\xa7\x00\xc6\x14\x8f\xcb\xf5\xde\
\x9e\xcd\xcd\xcdw)\x1a*\xe3\x03\x03\x03\x8d\x9e\xae/\
j^\x83C29\xa3C\xd7\xab\x8emI\x06*\xb4\
,\xfb\xa35\x91\xb8bbb\xe2\x1e\xa5\xb4RC\xc3\
k\xfb\xf7\xefO{\xfe\xf5\x07M0YwX\x87\x96\
)\xbew\x18\xe0/:\x86P,\x1a]mYVZ\
u\x07\x13\x89\xc4\xd6\x1c\xa6A\xc5d\xc7\x9c\xf4RM\
\xcab\xb4\x0c!Ug\xe5hgGG\x97\xcfu\x0f\
\xe9\x1c\xe1\xfe\xbe\xbe|\x02\xf2\xd1R\xd2\x19\xb8\xc50\
\xd5\xadc\x0a\x0f\x03\xfc@\xd5\xbfv\x1cgL\x07|\
0\x18\xfchhh(_\xdco\xb0\x84n\xdeT\x86\
`\xd7q\x86\xae\x02H\xa9\xb8\xc3&\x89\x88\xc8\xe2\xc5\
\x8f\x17\x91\xee\xe7\xcb\xe8\xcf}\xc0_e\xdc`L&\
F\xcb\xa1\x874\xdc\xe1\x8el\xa7wK\x85\x95t\xc1\
\xbb\xae\xfby*\x95Z\xa19\xf9\x862\xc1\xab\xca\x97\
\xd3\x12#\xbf\xab$x!\xc4t<\x9e\xdcH\xedi\
\x9b\xc6<w{;\xae\xcb\x17\xb73\xcd\xbfEc-\
w\xd4\x01|\x93\x0c\xd8\xaa\x96\xd2\xac\xcf\xb5\x08\xdf\xc7\
,\xd7\xfeM\xc1\xd7\x14z\xa2\xa3\xa3S\xd4a\x01t\
j\x19F\xf2\xa9\xd7\xdb\xcb\x05\xef\xf3\xf9_\x1b\x188\
\xbf\xb1\x0e\xe0\x07\xf1\xa4\xcd\x15\xda\x9d\xf9\x06\x89\xca\xc4\
\x84\x11x\xc7q\x0e.\xef\xean\xab\x03\xf8\x98\xcc0\
\xe9\x94\xd3\x15\xccd\xdfkPi\x91\xb1,\xebD[\
[\xea\xc2:\x80\x0f\xc8\x04\xad\xce|w\x14\x1c-\x12\
\x89\xfe\xc2D\xe8\x85\xc3\xcd\xb7\xd4\x09\xfc\x1e\xcd\xb9~\
^p\xf7[\x93\xad\x9b\x84\x10\xda\xe1kyb\xf6R\
\xe5\x02\x89\x1cj1\xd8\xf9\x0cpk\xfeK\xd4\xd2r\
\x9em\xdb\xc7M\xd2O\xd4\xb8DF\xf28`\x00~\
\x1f\x85\x8a\xaa]\xd7w\x7f\x85ro5-\x92\xd2h\
\x932\xc8Z\x90\xa2\x1a\x81\x11\x15#\xa9\xa6er\x0a\
\xed:\x15F\xdf\x93\x0e\x82R\xca\x19\xf5dj\xcd\x0a\
%\x0b\xb4\xdf\x17s\xabs\x1f<\x0e\xfc\xa8D\x96\x87\
\xb9\xb99\x93\x9d\xacY\xa9\xac\x87\xf6H\xd7zF\xb5\
\xc3b\x8a\xa4\xb3+P\x9bW\xcb\xf6\x02`d\x95~\
7_|@\x08q\xe2\x0cZ\x80\xe7L\xc1g\xe91\
\xbeYU\x95\x8e\xc7\x93k\x98\xff\x18a\xa1\x83\x7f\x80\
\x0a|C\xd4\x0c\xfc/\xeb:Fc-\xd7xd\xc6\
\xb5\xc0\xf1\x05\x08|RJ\xfb\x8ay\xa2\xdf\x01f#\
\x91\xd8\xee<\xcfz\xd0\xcf\xf9W\xb3\xed+\xa5\xe7M\
H\x84\xc3\xcd?\xeeYqn\xa1\xe3\xe4\x027Ki\
^/\xe0\x9f\x01\xb7Q\x87\xcf\xe6r\xdd\xd1\xdff]\
\xe9\x1a\xb5\x09\xe9\xd5\xb5\xb0\x80(\x06\xfcL\x06\x1b\xab\
\xf5\xe9\xec\x88\x0cf,(\xe0\xb9d3_\x84\xb8K\
\x86\xd9f\xcb\x00=+\x17t\xb7\x8c\xe19\x15\xbf\xe7\
U^\x0c\x01\x9c\xc3\xfc\xe7\xf3\xfd\xcc\x7f\x89\xd2\x01$\
\xa4s\x93-\xba:%#5G\xa4\x95\xf8!\xf0\x16\
\xf3\xe5;\xa3\x06\xb9Ae\xfa\x0a\xc4\xc3s\xf1\xb3\x0a\
\xea\x18\x00\x00\x00\x00IEND\xaeB`\x82\
"

qt_resource_name = b"\
\x00\x05\
\x00o\xa6S\
\x00i\
\x00c\x00o\x00n\x00s\
\x00\x11\
\x0f)y\xe7\
\x00d\
\x00e\x00l\x00t\x00o\x00d\x00o\x00n\x00-\x00l\x00o\x00g\x00o\x00.\x00p\x00n\x00g\
\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x02\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x10\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\
\x00\x00\x01\x92\x14\xa9,\xb7\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()



from typing import Callable, Dict, List, Tuple

import qtawesome as qta
from PySide6.QtCore import (
    QEasingCurve,
    QEvent,
    QPropertyAnimation,
    QSize,
    Qt,
    Signal,
)
from PySide6.QtGui import QEnterEvent
from PySide6.QtWidgets import (
    QFrame,
    QHBoxLayout,
    QLabel,
    QSizePolicy,
    QVBoxLayout,
    QWidget,
)

SIDEBAR_WIDTH_EXPANDED: int = 200
SIDEBAR_WIDTH_COLLAPSED: int = 50
BUTTON_HEIGHT: int = 50
ICON_SIZE: int = 20
ANIMATION_DURATION: int = 300


class SidebarButton(QWidget):
    """
    A custom sidebar button widget with an icon and label.

    This widget creates a button with an icon and label for use in a sidebar.
    It can be expanded or collapsed, showing or hiding the label text.

    Attributes:
        clicked (Signal): Signal emitted when the button is clicked.
    """

    clicked = Signal()

    def __init__(self, label: str, icon: str):
        """
        Initialize the SidebarButton.

        Args:
            label (str): The text to display on the button.
            icon (str): The name of the icon to display on the button.
        """
        super().__init__()
        self.label_text = label

        self.main_layout = QHBoxLayout(self)
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        self.main_layout.setSpacing(0)

        self.content_widget = QWidget()
        self.content_layout = QHBoxLayout(self.content_widget)
        self.content_layout.setContentsMargins(0, 0, 0, 0)
        self.content_layout.setSpacing(0)

        self.icon_label = QLabel()
        self.icon_label.setFixedSize(SIDEBAR_WIDTH_COLLAPSED, BUTTON_HEIGHT)
        self.icon_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.icon_label.setPixmap(
            qta.icon(icon, color="white").pixmap(QSize(ICON_SIZE, ICON_SIZE))
        )

        self.text_label = QLabel(label)
        # used in qss to set the spacing between icon and label
        self.text_label.setObjectName("text-label")
        # Initially hide the text
        self.text_label.hide()

        self.content_layout.addWidget(self.icon_label)
        self.content_layout.addWidget(self.text_label)
        self.content_layout.addStretch()

        self.main_layout.addWidget(self.content_widget)

        self.setFixedHeight(BUTTON_HEIGHT)
        self.setFixedWidth(
            SIDEBAR_WIDTH_COLLAPSED
        )  # Start with collapsed width
        self.setSizePolicy(
            QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed
        )

    def set_expanded(self, expanded):
        """
        Set the expanded state of the button.

        Args:
            expanded (bool): True to expand the button, False to collapse it.
        """
        if expanded:
            self.text_label.show()
            self.setFixedWidth(SIDEBAR_WIDTH_EXPANDED)
        else:
            self.text_label.hide()
            self.setFixedWidth(SIDEBAR_WIDTH_COLLAPSED)

    def enterEvent(self, event: QEnterEvent) -> None:
        """
        Handle the mouse enter event.

        Args:
            event (QEnterEvent): The enter event.
        """
        super().enterEvent(event)

    def leaveEvent(self, event: QEvent) -> None:
        """
        Handle the mouse leave event.

        Args:
            event (QEvent): The leave event.
        """
        super().leaveEvent(event)

    def mousePressEvent(self, event):
        """
        Handle the mouse press event.

        Args:
            event (QMouseEvent): The mouse press event.
        """
        self.clicked.emit()


class SideBar(QFrame):
    def __init__(
        self,
    ):
        super().__init__()
        self.sidebar_expanded = False
        self.setFixedWidth(SIDEBAR_WIDTH_COLLAPSED)
        self.sidebar_layout = QVBoxLayout(self)
        self.sidebar_layout.setContentsMargins(0, 10, 0, 0)
        self.sidebar_layout.setSpacing(10)

        # Add buttons to the sidebar
        self.create_sidebar_buttons()
        self.on_click("Toggle Sidebar", self.toggle_sidebar)

    def create_sidebar_buttons(self):
        buttons_params: List[Tuple[str, str]] = [
            ("Toggle Sidebar", "fa5s.bars"),
            ("Home", "fa5s.home"),
            ("Data", "fa5s.database"),
            ("Table", "fa.table"),
            ("Map", "fa5s.map"),
            ("Graph", "fa5s.chart-line"),
            # ("New File", "fa5s.file"),
            # ("Open File", "fa5s.folder-open"),
            # ("Search", "fa5s.search"),
            ("Full Screen", "fa5s.expand"),
            ("Settings", "fa5s.cog"),
        ]

        self.buttons: Dict[str, SidebarButton] = {}
        for label, icon in buttons_params:
            button = SidebarButton(label, icon)
            self.buttons[label] = button
            self.sidebar_layout.addWidget(button)

        # Add stretch at the end to push buttons to the top
        self.sidebar_layout.addStretch()

    def toggle_sidebar(self):
        width = (
            SIDEBAR_WIDTH_EXPANDED
            if not self.sidebar_expanded
            else SIDEBAR_WIDTH_COLLAPSED
        )
        self.sidebar_expanded = not self.sidebar_expanded

        self.animation = QPropertyAnimation(self, b"minimumWidth")
        self.animation.setDuration(ANIMATION_DURATION)
        self.animation.setStartValue(self.width())
        self.animation.setEndValue(width)
        self.animation.setEasingCurve(QEasingCurve.Type.InOutQuart)
        self.animation.start()

        for button in self.buttons.values():
            button.set_expanded(self.sidebar_expanded)

    def on_click(self, button_label: str, func: Callable):
        self.buttons[button_label].clicked.connect(func)



from PySide6 import QtGui
from PySide6.QtWidgets import QHBoxLayout, QMainWindow, QStackedWidget, QWidget

from pyside_demo.gui.data import DataWidget
from pyside_demo.gui.graph import GraphWidget
from pyside_demo.gui.home import HomeWidget
from pyside_demo.gui.map import MapWidget
from pyside_demo.gui.settings import SettingsWidget
from pyside_demo.gui.sidebar import SideBar
from pyside_demo.gui.table import TableWidget
from pyside_demo.gui.top_menu import create_menu_bar
from pyside_demo.model.table import TableModel
from pyside_demo.resources import rc_resources  # noqa: F401
from pyside_demo.resources.ui_mainwindow import Ui_MainWindow


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self._ui = Ui_MainWindow()
        self._ui.setupUi(self)
        self.setWindowTitle("PySide Demo")
        icon = QtGui.QIcon(":/icons/deltodon-logo.png")
        self.setWindowIcon(icon)

        self.setWindowTitle("PySide Demo")
        self.setGeometry(100, 100, 800, 600)
        create_menu_bar(self)

        # Create main widget and layout
        main_widget = QWidget()
        main_layout = QHBoxLayout(main_widget)
        main_layout.setContentsMargins(0, 0, 0, 0)
        main_layout.setSpacing(0)

        # Create sidebar
        self.sidebar = SideBar()
        self.sidebar.setObjectName("sidebar")
        sidebar_button_functions = [
            ("Home", self.show_home),
            ("Data", self.show_data),
            ("Table", self.show_table),
            ("Map", self.show_map),
            ("Graph", self.show_graph),
            # ("New File", self.new_file),
            # ("Open File", self.open_file),
            # ("Search", self.search_files),
            ("Full Screen", self.toggle_full_screen),
            ("Settings", self.show_settings),
        ]

        for label, func in sidebar_button_functions:
            self.sidebar.on_click(label, func)

        # Create table model
        self.table_model = TableModel()

        # Create content area
        self.content_area = QStackedWidget()

        # Create home widget
        self.home_dashboard = HomeWidget()
        self.content_area.addWidget(self.home_dashboard)

        # Create data widget
        self.data_widget = DataWidget(self.table_model)
        self.content_area.addWidget(self.data_widget)

        # Create table widget
        self.table_widget = TableWidget(self.table_model)
        self.content_area.addWidget(self.table_widget)

        # Create map widget
        self.map_widget = MapWidget()
        self.content_area.addWidget(self.map_widget)

        # Create graph widget
        self.graph_widget = GraphWidget()
        self.content_area.addWidget(self.graph_widget)

        # Create settings widget
        self.settings_widget = SettingsWidget()
        self.content_area.addWidget(self.settings_widget)

        # Add sidebar and content area to main layout
        main_layout.addWidget(self.sidebar)
        main_layout.addWidget(self.content_area)

        # Set central widget
        self.setCentralWidget(main_widget)

    def show_home(self):
        self.content_area.setCurrentWidget(self.home_dashboard)

    def show_data(self):
        self.content_area.setCurrentWidget(self.data_widget)

    def show_table(self):
        self.content_area.setCurrentWidget(self.table_widget)

    def show_map(self):
        self.content_area.setCurrentWidget(self.map_widget)

    def show_graph(self):
        self.content_area.setCurrentWidget(self.graph_widget)

    def show_settings(self):
        self.content_area.setCurrentWidget(self.settings_widget)

    def new_file(self):
        print("New File")

    def open_file(self):
        print("Open File")

    def search_files(self):
        print("Search")

    def toggle_full_screen(self):
        if self.isFullScreen():
            self.showNormal()
        else:
            self.showFullScreen()



from PySide6.QtWidgets import QMainWindow


def create_menu_bar(window: QMainWindow):
    menu_bar = window.menuBar()

    # File menu
    file_menu = menu_bar.addMenu("File")
    file_menu.addAction("New File")
    file_menu.addAction("Open File")
    file_menu.addAction("Save")
    file_menu.addAction("Exit")

    # Edit menu
    edit_menu = menu_bar.addMenu("Edit")
    edit_menu.addAction("Undo")
    edit_menu.addAction("Redo")
    edit_menu.addAction("Cut")
    edit_menu.addAction("Copy")
    edit_menu.addAction("Paste")

    # Selection menu
    selection_menu = menu_bar.addMenu("Selection")
    selection_menu.addAction("Select All")
    selection_menu.addAction("Expand Selection")

    # View menu
    view_menu = menu_bar.addMenu("View")
    view_menu.addAction("Toggle Sidebar")
    view_menu.addAction("Toggle Panel")

    # Go menu
    go_menu = menu_bar.addMenu("Go")
    go_menu.addAction("Go to File")
    go_menu.addAction("Go to Symbol")

    # Run menu
    run_menu = menu_bar.addMenu("Run")
    run_menu.addAction("Start Debugging")
    run_menu.addAction("Run Without Debugging")

    # Terminal menu
    terminal_menu = menu_bar.addMenu("Terminal")
    terminal_menu.addAction("New Terminal")
    terminal_menu.addAction("Split Terminal")

    # Help menu
    help_menu = menu_bar.addMenu("Help")
    help_menu.addAction("Welcome")
    help_menu.addAction("Documentation")
    help_menu.addAction("About")



from PySide6.QtWidgets import (
    QButtonGroup,
    QDialog,
    QLabel,
    QPushButton,
    QRadioButton,
    QVBoxLayout,
)


class ConflictResolutionDialog(QDialog):
    def __init__(self, item):
        super().__init__()
        self.item = item
        self.setWindowTitle("Resolve Conflict")
        self.setGeometry(200, 200, 300, 200)
        self.init_ui()

    def init_ui(self):
        layout = QVBoxLayout()

        layout.addWidget(
            QLabel(
                "Conflict detected for item: {}".format(
                    self.item.get("name", "UNKNOWN")
                )
            )
        )
        layout.addWidget(QLabel("Choose resolution:"))

        self.local_radio = QRadioButton("Keep local version")
        self.remote_radio = QRadioButton("Use remote version")

        button_group = QButtonGroup()
        button_group.addButton(self.local_radio)
        button_group.addButton(self.remote_radio)

        layout.addWidget(self.local_radio)
        layout.addWidget(self.remote_radio)

        resolve_button = QPushButton("Resolve")
        resolve_button.clicked.connect(self.accept)

        layout.addWidget(resolve_button)
        self.setLayout(layout)

    def get_resolution(self):
        if self.local_radio.isChecked():
            return "local"
        elif self.remote_radio.isChecked():
            return "remote"
        return None



from PySide6.QtCore import Qt
from PySide6.QtWidgets import (
    QHBoxLayout,
    QLineEdit,
    QListWidget,
    QListWidgetItem,
    QMessageBox,
    QPushButton,
    QTextEdit,
    QVBoxLayout,
    QWidget,
)

from pyside_demo.gui.dialog import ConflictResolutionDialog
from pyside_demo.model.table import TableModel


class DataWidget(QWidget):
    def __init__(self, model: TableModel):
        super().__init__()
        self.model = model

        self.main_layout = QHBoxLayout(self)

        # Left side: Add/Edit item form
        self.left_layout = QVBoxLayout()
        self.name_input = QLineEdit()
        self.name_input.setPlaceholderText("Item Name")
        self.description_input = QTextEdit()
        self.description_input.setPlaceholderText("Item Description")
        self.add_edit_button = QPushButton("Add Item")
        self.add_edit_button.clicked.connect(self.add_or_edit_item)

        self.left_layout.addWidget(self.name_input)
        self.left_layout.addWidget(self.description_input)
        self.left_layout.addWidget(self.add_edit_button)

        # Right side: Item list and sync button
        self.right_layout = QVBoxLayout()
        self.item_list = QListWidget()
        self.item_list.itemClicked.connect(self.load_item)
        sync_button = QPushButton("Sync with PostgreSQL")
        sync_button.clicked.connect(self.sync_with_postgresql)

        self.right_layout.addWidget(self.item_list)
        self.right_layout.addWidget(sync_button)

        self.main_layout.addLayout(self.left_layout)
        self.main_layout.addLayout(self.right_layout)

        self.setLayout(self.main_layout)

        self.load_items()

    def add_or_edit_item(self):
        name = self.name_input.text()
        description = self.description_input.toPlainText()

        if name and description:
            if self.add_edit_button.text() == "Add Item":
                self.model.add_item(name, description)
            else:
                selected_items = self.item_list.selectedItems()
                if selected_items:
                    item_id = selected_items[0].data(Qt.ItemDataRole.UserRole)
                    self.model.update_item(item_id, name, description)

            self.name_input.clear()
            self.description_input.clear()
            self.add_edit_button.setText("Add Item")
            self.load_items()
        else:
            QMessageBox.warning(
                self, "Input Error", "Please enter both name and description."
            )

    def load_items(self):
        self.item_list.clear()
        items = self.model.get_items()
        for item in items:
            list_item = QListWidgetItem(
                f"{item['name']} ({item['sync_status']})"
            )
            list_item.setData(Qt.ItemDataRole.UserRole, item["id"])
            self.item_list.addItem(list_item)

    def load_item(self, item):
        item_id = item.data(Qt.ItemDataRole.UserRole)
        db_item = self.model.get_item_by_id(item_id)
        if db_item:
            self.name_input.setText(str(db_item["name"]))
            self.description_input.setPlainText(str(db_item["description"]))
            self.add_edit_button.setText("Update Item")

    def sync_with_postgresql(self):
        self.model.sync_with_postgresql()
        self.resolve_conflicts()
        self.load_items()
        QMessageBox.information(
            self,
            "Sync Status",
            "Synchronization completed. Check console for details.",
        )

    def resolve_conflicts(self):
        conflict_items = self.model.get_conflict_items()

        for item in conflict_items:
            dialog = ConflictResolutionDialog(item)
            if dialog.exec_():
                resolution = dialog.get_resolution()
                self.model.resolve_conflict(item["id"], resolution)



SQL_CREATE_TABLE: str = """
CREATE TABLE IF NOT EXISTS items (
    id VARCHAR(255) PRIMARY KEY,
    name VARCHAR(255),
    description TEXT,
    created_at TIMESTAMP,
    updated_at TIMESTAMP,
    version INTEGER,
    sync_status VARCHAR(20)
)
"""

SQL_UPDATE_OR_INSERT_ITEM: str = """
INSERT INTO items (id, name, description, created_at,
                   updated_at, version, sync_status)
VALUES (%s, %s, %s, %s, %s, %s, %s)
ON CONFLICT (id) DO UPDATE
SET name = EXCLUDED.name,
    description = EXCLUDED.description,
    updated_at = EXCLUDED.updated_at,
    version = EXCLUDED.version,
    sync_status = EXCLUDED.sync_status
"""

SQL_FETCH_ITEMS: str = """
SELECT id,
       name,
       description,
       created_at,
       updated_at,
       version
FROM items
"""

SQL_CHECK_FOR_CONFLICTS: str = "SELECT version FROM items WHERE id = %s"

SQL_DELETE_ITEM: str = "DELETE FROM items WHERE id = %s"



from PySide6.QtCore import Qt
from PySide6.QtWidgets import QLabel, QVBoxLayout, QWidget


class HomeWidget(QWidget):
    def __init__(
        self,
    ):
        super().__init__()
        self.home_layout = QVBoxLayout(self)
        self.home_label = QLabel("TODO: Home Dashboard")
        self.home_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.home_layout.addWidget(self.home_label)



import sys

from dotenv import load_dotenv
from PySide6.QtWidgets import QApplication

from pyside_demo.gui.window import MainWindow
from pyside_demo.theme import set_theme


def main():
    load_dotenv()

    app = QApplication(sys.argv)
    window = MainWindow()

    set_theme(app, "dark")

    window.show()
    sys.exit(app.exec())


if __name__ == "__main__":
    main()



from PySide6.QtWidgets import (
    QAbstractItemView,
    QTableView,
    QVBoxLayout,
    QWidget,
)

from pyside_demo.model.table import TableModel


class TableWidget(QWidget):
    def __init__(self, model: TableModel):
        super().__init__()
        self.model = model
        self.main_layout = QVBoxLayout(self)

        # Table view
        self.table_view = QTableView()
        self.table_view.setModel(self.model)
        # Make the table read-only
        self.table_view.setEditTriggers(
            QAbstractItemView.EditTrigger.NoEditTriggers
        )

        # Adjust column widths
        self.table_view.setColumnWidth(0, 100)  # ID
        self.table_view.setColumnWidth(1, 120)  # Name
        self.table_view.setColumnWidth(2, 120)  # Description
        self.table_view.setColumnWidth(3, 120)  # Created At
        self.table_view.setColumnWidth(4, 120)  # Updated At
        self.table_view.setColumnWidth(5, 50)  # Version
        self.table_view.setColumnWidth(6, 70)  # Sync Status

        self.main_layout.addWidget(self.table_view)
        self.setLayout(self.main_layout)

    def refresh(self):
        self.model.refresh_data()



import folium
from PySide6.QtWebEngineWidgets import QWebEngineView


class MapWidget(QWebEngineView):
    def __init__(
        self,
    ):
        super().__init__()
        map = folium.Map(location=[0, 0], zoom_start=2)
        html_string = map.get_root().render()
        self.setHtml(html_string)



from typing import Any, Dict, List, Union

from PySide6.QtCore import (
    QAbstractTableModel,
    QModelIndex,
    QPersistentModelIndex,
    Qt,
)

from pyside_demo.db.database import Database, Item, SyncStatus


class TableModel(QAbstractTableModel):
    def __init__(self):
        super(TableModel, self).__init__()
        self.db = Database()
        self._data: List[Item] = []
        self._headers = [
            "ID",
            "Name",
            "Description",
            "Created At",
            "Updated At",
            "Version",
            "Sync Status",
        ]
        self.refresh_data()

    def refresh_data(self):
        self._data = self.db.get_items()
        self.layoutChanged.emit()

    def data(
        self,
        index: Union[QModelIndex, QPersistentModelIndex],
        role: int = Qt.ItemDataRole.DisplayRole,
    ) -> Any:
        if role == Qt.ItemDataRole.DisplayRole:
            item = self._data[index.row()]
            column = index.column()
            if column == 0:
                return str(item.id)
            elif column == 1:
                return str(item.name)
            elif column == 2:
                return str(item.description)
            elif column == 3:
                return item.created_at.strftime("%Y-%m-%d %H:%M:%S")
            elif column == 4:
                return item.updated_at.strftime("%Y-%m-%d %H:%M:%S")
            elif column == 5:
                return str(item.version)
            elif column == 6:
                return str(item.sync_status.value)
        return None

    def rowCount(
        self, parent: Union[QModelIndex, QPersistentModelIndex] = QModelIndex()
    ) -> int:
        return len(self._data)

    def columnCount(
        self, parent: Union[QModelIndex, QPersistentModelIndex] = QModelIndex()
    ) -> int:
        return len(self._headers)

    def headerData(
        self,
        section: int,
        orientation: Qt.Orientation,
        role: int = Qt.ItemDataRole.DisplayRole,
    ) -> Any:
        if (
            role == Qt.ItemDataRole.DisplayRole
            and orientation == Qt.Orientation.Horizontal
        ):
            return self._headers[section]
        return None

    def add_item(self, name: str, description: str):
        self.db.add_item(name, description)
        self.refresh_data()

    def update_item(self, item_id: str, name: str, description: str):
        self.db.update_item(item_id, name, description)
        self.refresh_data()

    def get_items(self) -> List[Dict[str, Any]]:
        return [self.item_to_dict(item) for item in self._data]

    def get_item_by_id(self, item_id: str) -> Union[Dict[str, Any], None]:
        for item in self._data:
            if item.id == item_id:
                return self.item_to_dict(item)
        return None

    def item_to_dict(self, item: Item) -> Dict[str, Any]:
        return {
            "id": item.id,
            "name": item.name,
            "description": item.description,
            "created_at": item.created_at,
            "updated_at": item.updated_at,
            "version": item.version,
            "sync_status": item.sync_status.value,
        }

    def sync_with_postgresql(self):
        self.db.sync_with_postgresql()
        self.refresh_data()

    def get_conflict_items(self) -> List[Dict[str, Any]]:
        return [
            self.item_to_dict(item)
            for item in self._data
            if item.sync_status == SyncStatus.CONFLICT
        ]

    def resolve_conflict(self, item_id: str, resolution_choice: str):
        self.db.resolve_conflict(item_id, resolution_choice)
        self.refresh_data()



from PySide6.QtCore import Qt
from PySide6.QtWidgets import QLabel, QVBoxLayout, QWidget


class SettingsWidget(QWidget):
    def __init__(
        self,
    ):
        super().__init__()
        self.settings_layout = QVBoxLayout(self)
        self.settings_label = QLabel("TODO: Settings Widget")
        self.settings_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.settings_layout.addWidget(self.settings_label)



"""This module demonstrates 2D plotting capabilities in `PyQtGraph <https://www.pyqtgraph.org/>`_

Source:
    This functionality is implemented from `PyQtGraph example <https://pyqtgraph.readthedocs.io/en/latest/getting_started/introduction.html#examples>`_:

    ``.venv/Lib/site-packages/pyqtgraph/examples/Plotting.py``

    that can also be accessed by running::

        $ poetry run python -m pyqtgraph.examples

    and selecting ``Basic Plotting``.

The module showcases various 2D plotting features available in pyqtgraph.
All plots can be panned/scaled by dragging with left/right mouse buttons.
Right-clicking on any plot displays a context menu.

Original source code was updated to pass mypy test.
"""  # noqa: E501

from typing import Union, overload

import numpy as np
import pyqtgraph as pg
from PySide6.QtCore import QRect
from PySide6.QtGui import QBitmap, QPolygon, QRegion
from PySide6.QtWidgets import QGridLayout, QWidget


class GraphWidget(QWidget):
    def __init__(self):
        super().__init__()

        layout = QGridLayout()
        self.setLayout(layout)

        # Enable antialiasing for prettier plots
        pg.setConfigOptions(antialias=True)

        # Create plots
        plot_1 = pg.PlotWidget(title="Basic array plotting")
        plot_1.plot(y=np.random.normal(size=100))
        layout.addWidget(plot_1, 0, 0)

        plot_2 = pg.PlotWidget(title="Multiple curves")
        plot_2.plot(
            np.random.normal(size=100), pen=(255, 0, 0), name="Red curve"
        )
        plot_2.plot(
            np.random.normal(size=110) + 5, pen=(0, 255, 0), name="Green curve"
        )
        plot_2.plot(
            np.random.normal(size=120) + 10, pen=(0, 0, 255), name="Blue curve"
        )
        layout.addWidget(plot_2, 0, 1)

        plot_3 = pg.PlotWidget(title="Drawing with points")
        plot_3.plot(
            np.random.normal(size=100),
            pen=(200, 200, 200),
            symbolBrush=(255, 0, 0),
            symbolPen="w",
        )
        layout.addWidget(plot_3, 0, 2)

        plot_4 = pg.PlotWidget(title="Parametric, grid enabled")
        x = np.cos(np.linspace(0, 2 * np.pi, 1000))
        y = np.sin(np.linspace(0, 4 * np.pi, 1000))
        plot_4.plot(x, y)
        plot_4.showGrid(x=True, y=True)
        layout.addWidget(plot_4, 1, 0)

        plot_5 = pg.PlotWidget(title="Scatter plot, axis labels, log scale")
        x = np.random.normal(size=1000) * 1e-5
        y = x * 1000 + 0.005 * np.random.normal(size=1000)
        y -= y.min() - 1.0
        mask = x > 1e-15
        x = x[mask]
        y = y[mask]
        plot_5.plot(
            x,
            y,
            pen=None,
            symbol="t",
            symbolPen=None,
            symbolSize=10,
            symbolBrush=(100, 100, 255, 50),
        )
        plot_5.setLabel("left", "Y Axis", units="A")
        plot_5.setLabel("bottom", "Y Axis", units="s")
        plot_5.setLogMode(x=True, y=False)
        layout.addWidget(plot_5, 1, 1)

        plot_6 = pg.PlotWidget(title="Updating plot")
        self.curve = plot_6.plot(pen="y")
        self.data = np.random.normal(size=(10, 1000))
        self.ptr = 0
        layout.addWidget(plot_6, 1, 2)

        plot_7 = pg.PlotWidget(title="Filled plot, axis disabled")
        y = np.sin(np.linspace(0, 10, 1000)) + np.random.normal(
            size=1000, scale=0.1
        )
        plot_7.plot(y, fillLevel=-0.3, brush=(50, 50, 200, 100))
        plot_7.showAxis("bottom", False)
        layout.addWidget(plot_7, 2, 0)

        plot_8 = pg.PlotWidget(title="Region Selection")
        x2 = np.linspace(-100, 100, 1000)
        data2 = np.sin(x2) / x2
        plot_8.plot(data2, pen=(255, 255, 255, 200))
        self.lr = pg.LinearRegionItem([400, 700])
        self.lr.setZValue(-10)
        plot_8.addItem(self.lr)
        layout.addWidget(plot_8, 2, 1)

        plot_9 = pg.PlotWidget(title="Zoom on selected region")
        plot_9.plot(data2)
        layout.addWidget(plot_9, 2, 2)

        def updatePlot():
            plot_9.setXRange(*self.lr.getRegion(), padding=0)

        def updateRegion():
            self.lr.setRegion(plot_9.getViewBox().viewRange()[0])

        self.lr.sigRegionChanged.connect(updatePlot)
        plot_9.sigXRangeChanged.connect(updateRegion)
        updatePlot()

        # Setup timer for updating plot
        self.timer = pg.QtCore.QTimer()
        self.timer.timeout.connect(self.update_plot)
        self.timer.start(50)

    def update_plot(self):
        self.curve.setData(self.data[self.ptr % 10])
        self.ptr += 1

    @overload
    def update(self) -> None: ...

    @overload
    def update(self, arg__1: QRect) -> None: ...

    @overload
    def update(
        self, arg__1: Union[QRegion, QBitmap, QPolygon, QRect]
    ) -> None: ...

    @overload
    def update(self, x: int, y: int, w: int, h: int) -> None: ...

    def update(self, *args, **kwargs) -> None:
        super().update(*args, **kwargs)
        self.update_plot()



import os

from PySide6.QtWidgets import QApplication

THEME_DIR: str = os.path.dirname(os.path.abspath(__file__))


def set_theme(app: QApplication, theme_name: str):
    with open(f"{THEME_DIR}/{theme_name}.qss", "r") as f:
        theme: str = f.read()
        app.setStyleSheet(theme)



import os
import uuid
from datetime import datetime
from enum import Enum as PyEnum
from typing import Any, Optional

import psycopg2
import requests
from sqlalchemy import Column, DateTime
from sqlalchemy import Enum as SQLAlchemyEnum
from sqlalchemy import Integer, String, create_engine
from sqlalchemy.orm import DeclarativeBase, sessionmaker

from pyside_demo.db.sql import (
    SQL_CHECK_FOR_CONFLICTS,
    SQL_CREATE_TABLE,
    SQL_DELETE_ITEM,
    SQL_FETCH_ITEMS,
    SQL_UPDATE_OR_INSERT_ITEM,
)


class Base(DeclarativeBase):
    """
    Base class for declarative SQLAlchemy models.

    This class serves as the base for all database models in the application.
    It inherits from SQLAlchemy's DeclarativeBase, providing the necessary
    functionality for declarative model definitions.
    """

    pass


SQLITE_FILE_NAME: str = "local.db"


class SyncStatus(str, PyEnum):
    SYNCED = "synced"
    MODIFIED = "modified"
    DELETED = "deleted"
    CONFLICT = "conflict"


class Item(Base):
    __tablename__ = "items"

    id: Any = Column(
        String, primary_key=True, default=lambda: str(uuid.uuid4())
    )
    name: Any = Column(String)
    description: Any = Column(String)
    created_at: Any = Column(DateTime, default=datetime.utcnow)
    updated_at: Any = Column(
        DateTime, default=datetime.utcnow, onupdate=datetime.utcnow
    )
    version: Any = Column(Integer, default=1)
    sync_status: Any = Column(
        SQLAlchemyEnum(SyncStatus), default=SyncStatus.MODIFIED
    )


class Database:
    def __init__(self):
        self.local_engine = create_engine(f"sqlite:///{SQLITE_FILE_NAME}")
        Base.metadata.create_all(self.local_engine)
        self.Session = sessionmaker(bind=self.local_engine)

    def add_item(self, name, description):
        session = self.Session()
        new_item = Item(name=name, description=description)
        session.add(new_item)
        session.commit()
        session.close()

    def update_item(self, item_id, name, description):
        session = self.Session()
        item = session.query(Item).filter_by(id=item_id).first()
        if item:
            item.name = name
            item.description = description
            item.version += 1
            item.sync_status = SyncStatus.MODIFIED
            session.commit()
        session.close()

    def set_conflict(self, item_id):
        session = self.Session()
        item = session.query(Item).filter_by(id=item_id).first()
        if item:
            item.sync_status = SyncStatus.CONFLICT
            session.commit()
        session.close()

    def delete_item(self, item_id):
        session = self.Session()
        item = session.query(Item).filter_by(id=item_id).first()
        if item:
            item.sync_status = SyncStatus.DELETED
            session.commit()
        session.close()

    def get_items(self):
        session = self.Session()
        items = (
            session.query(Item)
            .filter(Item.sync_status != SyncStatus.DELETED)
            .all()
        )
        session.close()
        return items

    def is_online(self):
        try:
            requests.get("https://www.google.com", timeout=5)
            return True
        except requests.ConnectionError:
            return False

    def sync_with_postgresql(self):
        if not self.is_online():
            print("Not online, can't sync with PostgreSQL")
            return

        with self._get_pg_connection() as conn:
            with conn.cursor() as cur:
                self._create_table_if_not_exists(cur)
                self._sync_local_to_remote(cur)
                self._sync_remote_to_local(cur)

            print("Sync with PostgreSQL completed successfully")

    def resolve_conflict(self, item_id, resolution_choice):
        # TODO: make this functionality more robust
        session = self.Session()
        # pg_item = get_pg_item()
        item = session.query(Item).filter_by(id=item_id).first()
        if item and item.sync_status == SyncStatus.CONFLICT:
            if resolution_choice == "local":
                item.sync_status = SyncStatus.MODIFIED
                # new_item = item
                # new_item.version = max(pg_item.version + 1, item.version)
                # update_pg_item(item)
            elif resolution_choice == "remote":
                # Fetch the latest version
                # from PostgreSQL and update local
                pass
            session.commit()
        session.close()

    def _get_pg_connection(self):
        pg_host: Optional[str] = os.getenv("DB_HOST")
        pg_database: Optional[str] = os.getenv("DB_NAME")
        pg_user: Optional[str] = os.getenv("DB_USER")
        pg_password: Optional[str] = os.getenv("DB_PASSWORD")

        return psycopg2.connect(
            host=pg_host,
            database=pg_database,
            user=pg_user,
            password=pg_password,
        )

    def _create_table_if_not_exists(self, cur):
        cur.execute(SQL_CREATE_TABLE)

    def _sync_local_to_remote(self, cur):
        local_items = self.get_items()
        for item in local_items:
            if item.sync_status == SyncStatus.MODIFIED:
                self._handle_modified_item(cur, item)
            elif item.sync_status == SyncStatus.DELETED:
                self._handle_deleted_item(cur, item)

    def _handle_modified_item(self, cur, item):
        if self._check_for_conflict(cur, item):
            self.set_conflict(item.id)
        else:
            self._update_or_insert_item(cur, item)
            item.sync_status = SyncStatus.SYNCED

    def _check_for_conflict(self, cur, item):
        cur.execute(SQL_CHECK_FOR_CONFLICTS, (item.id,))
        result = cur.fetchone()
        return result and result[0] > item.version

    def _update_or_insert_item(self, cur, item):
        cur.execute(
            SQL_UPDATE_OR_INSERT_ITEM,
            (
                item.id,
                item.name,
                item.description,
                item.created_at,
                item.updated_at,
                item.version,
                "synced",
            ),
        )

    def _handle_deleted_item(self, cur, item):
        cur.execute(SQL_DELETE_ITEM, (item.id,))

    def _sync_remote_to_local(self, cur):
        cur.execute(SQL_FETCH_ITEMS)
        pg_items = cur.fetchall()

        with self.Session() as session:
            for pg_item in pg_items:
                local_item = (
                    session.query(Item).filter_by(id=pg_item[0]).first()
                )
                if not local_item:
                    self._add_remote_item_to_local(session, pg_item)
            session.commit()

    def _add_remote_item_to_local(self, session, pg_item):
        new_item = Item(
            id=pg_item[0],
            name=pg_item[1],
            description=pg_item[2],
            created_at=pg_item[3],
            updated_at=pg_item[4],
            version=pg_item[5],
            sync_status=SyncStatus.SYNCED,
        )
        session.add(new_item)
